@@ -1,12 +1,16 @@
-// Copyright 2021 The AccessKit Authors. All rights reserved.
+// Copyright 2022 The AccessKit Authors. All rights reserved.
 // Licensed under the Apache License, Version 2.0 (found in
 // the LICENSE-APACHE file) or the MIT license (found in
 // the LICENSE-MIT file), at your option.
 
-use accesskit::{ActionHandler, NodeId, TreeUpdate};
-use lazy_static::lazy_static;
-use parking_lot::{const_mutex, Condvar, Mutex};
-use std::{cell::RefCell, rc::Rc, sync::Arc, time::Duration};
+use accesskit::{ActionHandler, ActivationHandler};
+use once_cell::sync::Lazy;
+use std::{
+    cell::RefCell,
+    sync::{Arc, Condvar, Mutex},
+    thread,
+    time::Duration,
+};
 use windows as Windows;
 use windows::{
     core::*,
@@ -18,146 +22,111 @@ use windows::{
     },
 };
 
-use super::Adapter;
+use crate::window_handle::WindowHandle;
+
+use super::{
+    context::{ActionHandlerNoMut, ActionHandlerWrapper},
+    Adapter,
+};
 
 const DEFAULT_TIMEOUT: Duration = Duration::from_secs(5);
 
-lazy_static! {
-    static ref WIN32_INSTANCE: HINSTANCE = {
-        let instance = unsafe { GetModuleHandleW(None) };
-        if instance.0 == 0 {
-            let result: Result<()> = Err(Error::from_win32());
-            result.unwrap();
-        }
-        instance
+static WINDOW_CLASS_ATOM: Lazy<u16> = Lazy::new(|| {
+    let class_name = w!("AccessKitTest");
+
+    let wc = WNDCLASSW {
+        hCursor: unsafe { LoadCursorW(None, IDC_ARROW) }.unwrap(),
+        hInstance: unsafe { GetModuleHandleW(None) }.unwrap().into(),
+        lpszClassName: class_name,
+        style: CS_HREDRAW | CS_VREDRAW,
+        lpfnWndProc: Some(wndproc),
+        ..Default::default()
     };
 
-    static ref DEFAULT_CURSOR: HCURSOR = {
-        let cursor = unsafe { LoadCursorW(None, IDC_ARROW) };
-        if cursor.0 == 0 {
-            let result: Result<()> = Err(Error::from_win32());
-            result.unwrap();
-        }
-        cursor
-    };
-
-    static ref WINDOW_CLASS_ATOM: u16 = {
-        // The following is a combination of the implementation of
-        // IntoParam<PWSTR> and the class registration function from winit.
-        let class_name_wsz: Vec<_> = "AccessKitTest"
-            .encode_utf16()
-            .chain(std::iter::once(0))
-            .collect();
-
-        let wc = WNDCLASSW {
-            hCursor: *DEFAULT_CURSOR,
-            hInstance: *WIN32_INSTANCE,
-            lpszClassName: PWSTR(class_name_wsz.as_ptr() as _),
-            style: CS_HREDRAW | CS_VREDRAW,
-            lpfnWndProc: Some(wndproc),
-            ..Default::default()
-        };
-
-        let atom = unsafe { RegisterClassW(&wc) };
-        if atom == 0 {
-            let result: Result<()> = Err(Error::from_win32());
-            result.unwrap();
-        }
-        atom
-    };
-}
-
-struct InnerWindowState {
-    focus: NodeId,
-    is_window_focused: bool,
-}
+    let atom = unsafe { RegisterClassW(&wc) };
+    if atom == 0 {
+        panic!("{}", Error::from_win32());
+    }
+    atom
+});
 
 struct WindowState {
-    adapter: Adapter,
-    inner_state: Rc<RefCell<InnerWindowState>>,
+    activation_handler: RefCell<Box<dyn ActivationHandler>>,
+    adapter: RefCell<Adapter>,
 }
 
 unsafe fn get_window_state(window: HWND) -> *const WindowState {
     GetWindowLongPtrW(window, GWLP_USERDATA) as _
 }
 
-fn update_focus(window: HWND, is_window_focused: bool) {
-    let window_state = unsafe { &*get_window_state(window) };
-    let mut inner_state = window_state.inner_state.borrow_mut();
-    inner_state.is_window_focused = is_window_focused;
-    let focus = inner_state.focus;
-    drop(inner_state);
-    let events = window_state.adapter.update_if_active(|| TreeUpdate {
-        nodes: vec![],
-        tree: None,
-        focus: is_window_focused.then(|| focus),
-    });
-    events.raise();
+fn update_window_focus_state(window: HWND, is_focused: bool) {
+    let state = unsafe { &*get_window_state(window) };
+    let mut adapter = state.adapter.borrow_mut();
+    if let Some(events) = adapter.update_window_focus_state(is_focused) {
+        events.raise();
+    }
 }
 
-struct WindowCreateParams(TreeUpdate, NodeId, Box<dyn ActionHandler>);
+struct WindowCreateParams {
+    activation_handler: Box<dyn ActivationHandler>,
+    action_handler: Arc<dyn ActionHandlerNoMut + Send + Sync>,
+}
 
 extern "system" fn wndproc(window: HWND, message: u32, wparam: WPARAM, lparam: LPARAM) -> LRESULT {
-    match message as u32 {
+    match message {
         WM_NCCREATE => {
             let create_struct: &CREATESTRUCTW = unsafe { &mut *(lparam.0 as *mut _) };
             let create_params: Box<WindowCreateParams> =
                 unsafe { Box::from_raw(create_struct.lpCreateParams as _) };
-            let WindowCreateParams(initial_state, initial_focus, action_handler) = *create_params;
-            let inner_state = Rc::new(RefCell::new(InnerWindowState {
-                focus: initial_focus,
-                is_window_focused: false,
-            }));
-            let inner_state_for_tree_init = inner_state.clone();
+            let WindowCreateParams {
+                activation_handler,
+                action_handler,
+            } = *create_params;
+            let adapter = Adapter::with_wrapped_action_handler(window, false, action_handler);
             let state = Box::new(WindowState {
-                adapter: Adapter::new(
-                    window,
-                    Box::new(move || {
-                        let mut result = initial_state;
-                        let state = inner_state_for_tree_init.borrow();
-                        result.focus = state.is_window_focused.then(|| state.focus);
-                        result
-                    }),
-                    action_handler,
-                ),
-                inner_state,
+                activation_handler: RefCell::new(activation_handler),
+                adapter: RefCell::new(adapter),
             });
             unsafe { SetWindowLongPtrW(window, GWLP_USERDATA, Box::into_raw(state) as _) };
             unsafe { DefWindowProcW(window, message, wparam, lparam) }
         }
         WM_PAINT => {
-            unsafe { ValidateRect(window, std::ptr::null()) }.unwrap();
+            unsafe { ValidateRect(Some(window), None) }.unwrap();
             LRESULT(0)
         }
         WM_DESTROY => {
             let ptr = unsafe { SetWindowLongPtrW(window, GWLP_USERDATA, 0) };
             if ptr != 0 {
-                let _dropped: Box<WindowState> = unsafe { Box::from_raw(ptr as _) };
+                drop(unsafe { Box::<WindowState>::from_raw(ptr as _) });
             }
             unsafe { PostQuitMessage(0) };
             LRESULT(0)
         }
         WM_GETOBJECT => {
-            let window_state = unsafe { get_window_state(window) };
-            if window_state.is_null() {
+            let state_ptr = unsafe { get_window_state(window) };
+            if state_ptr.is_null() {
                 // We need to be prepared to gracefully handle WM_GETOBJECT
                 // while the window is being destroyed; this can happen if
                 // the thread is using a COM STA.
                 return unsafe { DefWindowProcW(window, message, wparam, lparam) };
             }
-            let window_state = unsafe { &*window_state };
-            let result = window_state.adapter.handle_wm_getobject(wparam, lparam);
+            let state = unsafe { &*state_ptr };
+            let mut adapter = state.adapter.borrow_mut();
+            let mut activation_handler = state.activation_handler.borrow_mut();
+            let result = adapter.handle_wm_getobject(wparam, lparam, &mut **activation_handler);
+            drop(activation_handler);
+            drop(adapter);
             result.map_or_else(
                 || unsafe { DefWindowProcW(window, message, wparam, lparam) },
                 |result| result.into(),
             )
         }
         WM_SETFOCUS | WM_EXITMENULOOP | WM_EXITSIZEMOVE => {
-            update_focus(window, true);
+            update_window_focus_state(window, true);
             LRESULT(0)
         }
         WM_KILLFOCUS | WM_ENTERMENULOOP | WM_ENTERSIZEMOVE => {
-            update_focus(window, false);
+            update_window_focus_state(window, false);
             LRESULT(0)
         }
         _ => unsafe { DefWindowProcW(window, message, wparam, lparam) },
@@ -166,21 +135,20 @@ extern "system" fn wndproc(window: HWND, message: u32, wparam: WPARAM, lparam: L
 
 fn create_window(
     title: &str,
-    initial_state: TreeUpdate,
-    initial_focus: NodeId,
-    action_handler: Box<dyn ActionHandler>,
+    activation_handler: impl 'static + ActivationHandler,
+    action_handler: impl 'static + ActionHandler + Send,
 ) -> Result<HWND> {
-    let create_params = Box::new(WindowCreateParams(
-        initial_state,
-        initial_focus,
-        action_handler,
-    ));
+    let create_params = Box::new(WindowCreateParams {
+        activation_handler: Box::new(activation_handler),
+        action_handler: Arc::new(ActionHandlerWrapper::new(action_handler)),
+    });
+    let module = HINSTANCE::from(unsafe { GetModuleHandleW(None)? });
 
     let window = unsafe {
         CreateWindowExW(
             Default::default(),
-            PWSTR(*WINDOW_CLASS_ATOM as usize as _),
-            title,
+            PCWSTR(*WINDOW_CLASS_ATOM as usize as _),
+            &HSTRING::from(title),
             WS_OVERLAPPEDWINDOW,
             CW_USEDEFAULT,
             CW_USEDEFAULT,
@@ -188,11 +156,11 @@ fn create_window(
             CW_USEDEFAULT,
             None,
             None,
-            *WIN32_INSTANCE,
-            Box::into_raw(create_params) as _,
-        )
+            Some(module),
+            Some(Box::into_raw(create_params) as _),
+        )?
     };
-    if window.0 == 0 {
+    if window.is_invalid() {
         return Err(Error::from_win32());
     }
 
@@ -201,36 +169,35 @@ fn create_window(
 
 pub(crate) struct Scope {
     pub(crate) uia: IUIAutomation,
-    pub(crate) window: HWND,
+    pub(crate) window: WindowHandle,
 }
 
 impl Scope {
     pub(crate) fn show_and_focus_window(&self) {
-        unsafe { ShowWindow(self.window, SW_SHOW) };
-        unsafe { SetForegroundWindow(self.window) };
+        let _ = unsafe { ShowWindow(self.window.0, SW_SHOW) };
+        let _ = unsafe { SetForegroundWindow(self.window.0) };
     }
 }
 
 // It's not safe to run these UI-related tests concurrently.
-static MUTEX: Mutex<()> = const_mutex(());
+pub(crate) static MUTEX: Mutex<()> = Mutex::new(());
 
 pub(crate) fn scope<F>(
     window_title: &str,
-    initial_state: TreeUpdate,
-    initial_focus: NodeId,
-    action_handler: Box<dyn ActionHandler>,
+    activation_handler: impl 'static + ActivationHandler + Send,
+    action_handler: impl 'static + ActionHandler + Send,
     f: F,
 ) -> Result<()>
 where
     F: FnOnce(&Scope) -> Result<()>,
 {
-    let _lock_guard = MUTEX.lock();
+    let _lock_guard = MUTEX.lock().unwrap();
 
-    let window_mutex: Mutex<Option<HWND>> = Mutex::new(None);
+    let window_mutex: Mutex<Option<WindowHandle>> = Mutex::new(None);
     let window_cv = Condvar::new();
 
-    crossbeam_utils::thread::scope(|thread_scope| {
-        thread_scope.spawn(|_| {
+    thread::scope(|thread_scope| {
+        thread_scope.spawn(|| {
             // We explicitly don't want to initialize COM on the provider thread,
             // because we want to make sure that the provider side of UIA works
             // even if COM is never initialized on the provider thread
@@ -238,32 +205,33 @@ where
             // initialized after the window is shown (as is the case,
             // at least on some Windows 10 machines, due to IME support).
 
-            let window =
-                create_window(window_title, initial_state, initial_focus, action_handler).unwrap();
+            let window = create_window(window_title, activation_handler, action_handler).unwrap();
 
             {
-                let mut state = window_mutex.lock();
-                *state = Some(window);
+                let mut state = window_mutex.lock().unwrap();
+                *state = Some(window.into());
                 window_cv.notify_one();
             }
 
             let mut message = MSG::default();
-            while unsafe { GetMessageW(&mut message, HWND(0), 0, 0) }.into() {
-                unsafe { TranslateMessage(&message) };
+            while unsafe { GetMessageW(&mut message, None, 0, 0) }.into() {
+                let _ = unsafe { TranslateMessage(&message) };
                 unsafe { DispatchMessageW(&message) };
             }
         });
 
         let window = {
-            let mut state = window_mutex.lock();
-            if state.is_none() {
-                window_cv.wait(&mut state);
-            }
+            let state = window_mutex.lock().unwrap();
+            let mut state = if state.is_none() {
+                window_cv.wait(state).unwrap()
+            } else {
+                state
+            };
             state.take().unwrap()
         };
 
         let _window_guard = scopeguard::guard((), |_| {
-            unsafe { PostMessageW(window, WM_CLOSE, WPARAM(0), LPARAM(0)) }.unwrap()
+            unsafe { PostMessageW(Some(window.0), WM_CLOSE, WPARAM(0), LPARAM(0)) }.unwrap()
         });
 
         // We must initialize COM before creating the UIA client. The MTA option
@@ -278,7 +246,7 @@ where
         // of UIA works even if it is set up in an environment where COM
         // has not been initialized, and that this sequence of events
         // doesn't prevent the UIA client from working.
-        unsafe { CoInitializeEx(std::ptr::null_mut(), COINIT_MULTITHREADED) }.unwrap();
+        unsafe { CoInitializeEx(None, COINIT_MULTITHREADED) }.unwrap();
         let _com_guard = scopeguard::guard((), |_| unsafe { CoUninitialize() });
 
         let uia: IUIAutomation =
@@ -287,11 +255,15 @@ where
         let s = Scope { uia, window };
         f(&s)
     })
-    .unwrap()
 }
 
+/// This must only be used to wrap UIA elements returned by a UIA client
+/// that was created in the MTA. Those are safe to send between threads.
+struct SendableUiaElement(IUIAutomationElement);
+unsafe impl Send for SendableUiaElement {}
+
 pub(crate) struct ReceivedFocusEvent {
-    mutex: Mutex<Option<IUIAutomationElement>>,
+    mutex: Mutex<Option<SendableUiaElement>>,
     cv: Condvar,
 }
 
@@ -307,21 +279,22 @@ impl ReceivedFocusEvent {
     where
         F: Fn(&IUIAutomationElement) -> bool,
     {
-        let mut received = self.mutex.lock();
+        let mut received = self.mutex.lock().unwrap();
         loop {
-            if let Some(element) = received.take() {
+            if let Some(SendableUiaElement(element)) = received.take() {
                 if f(&element) {
                     return element;
                 }
             }
-            let result = self.cv.wait_for(&mut received, DEFAULT_TIMEOUT);
+            let (lock, result) = self.cv.wait_timeout(received, DEFAULT_TIMEOUT).unwrap();
             assert!(!result.timed_out());
+            received = lock;
         }
     }
 
     fn put(&self, element: IUIAutomationElement) {
-        let mut received = self.mutex.lock();
-        *received = Some(element);
+        let mut received = self.mutex.lock().unwrap();
+        *received = Some(SendableUiaElement(element));
         self.cv.notify_one();
     }
 }
@@ -330,8 +303,11 @@ impl ReceivedFocusEvent {
 pub(crate) struct FocusEventHandler {
     received: Arc<ReceivedFocusEvent>,
 }
+// Because we create a UIA client in the COM MTA, this event handler
+// _will_ be called from a different thread, and possibly multiple threads
+// at once.
+static_assertions::assert_impl_all!(FocusEventHandler: Send, Sync);
 
-#[allow(non_snake_case)]
 impl FocusEventHandler {
     #[allow(clippy::new_ret_no_self)] // it does return self, but wrapped
     pub(crate) fn new() -> (
@@ -341,17 +317,21 @@ impl FocusEventHandler {
         let received = ReceivedFocusEvent::new();
         (
             Self {
-                received: received.clone(),
+                received: Arc::clone(&received),
             }
             .into(),
             received,
         )
     }
+}
 
-    fn HandleFocusChangedEvent(&self, sender: &Option<IUIAutomationElement>) -> Result<()> {
-        self.received.put(sender.as_ref().unwrap().clone());
+#[allow(non_snake_case)]
+impl IUIAutomationFocusChangedEventHandler_Impl for FocusEventHandler_Impl {
+    fn HandleFocusChangedEvent(&self, sender: Ref<IUIAutomationElement>) -> Result<()> {
+        self.received.put(sender.unwrap().clone());
         Ok(())
     }
 }
 
 mod simple;
+mod subclassed;
