@@ -1,12 +1,10 @@
-// Copyright 2021 The AccessKit Authors. All rights reserved.
+// Copyright 2022 The AccessKit Authors. All rights reserved.
 // Licensed under the Apache License, Version 2.0 (found in
 // the LICENSE-APACHE file) or the MIT license (found in
 // the LICENSE-MIT file), at your option.
 
-use std::{convert::TryInto, num::NonZeroU64};
-
 use accesskit::{
-    ActionHandler, ActionRequest, Node, NodeId, Role, StringEncoding, Tree, TreeId, TreeUpdate,
+    Action, ActionHandler, ActionRequest, ActivationHandler, Node, NodeId, Role, Tree, TreeUpdate,
 };
 use windows::{core::*, Win32::UI::Accessibility::*};
 
@@ -14,41 +12,45 @@ use super::*;
 
 const WINDOW_TITLE: &str = "Simple test";
 
-const WINDOW_ID: NodeId = NodeId(unsafe { NonZeroU64::new_unchecked(1) });
-const BUTTON_1_ID: NodeId = NodeId(unsafe { NonZeroU64::new_unchecked(2) });
-const BUTTON_2_ID: NodeId = NodeId(unsafe { NonZeroU64::new_unchecked(3) });
+const WINDOW_ID: NodeId = NodeId(0);
+const BUTTON_1_ID: NodeId = NodeId(1);
+const BUTTON_2_ID: NodeId = NodeId(2);
 
-fn make_button(id: NodeId, name: &str) -> Node {
-    Node {
-        name: Some(name.into()),
-        focusable: true,
-        ..Node::new(id, Role::Button)
-    }
+fn make_button(label: &str) -> Node {
+    let mut node = Node::new(Role::Button);
+    node.set_label(label);
+    node.add_action(Action::Focus);
+    node
 }
 
 fn get_initial_state() -> TreeUpdate {
-    let root = Node {
-        children: vec![BUTTON_1_ID, BUTTON_2_ID],
-        name: Some(WINDOW_TITLE.into()),
-        ..Node::new(WINDOW_ID, Role::Window)
-    };
-    let button_1 = make_button(BUTTON_1_ID, "Button 1");
-    let button_2 = make_button(BUTTON_2_ID, "Button 2");
+    let mut root = Node::new(Role::Window);
+    root.set_children(vec![BUTTON_1_ID, BUTTON_2_ID]);
+    let button_1 = make_button("Button 1");
+    let button_2 = make_button("Button 2");
     TreeUpdate {
-        nodes: vec![root, button_1, button_2],
-        tree: Some(Tree::new(
-            TreeId("test".into()),
-            WINDOW_ID,
-            StringEncoding::Utf8,
-        )),
-        focus: None,
+        nodes: vec![
+            (WINDOW_ID, root),
+            (BUTTON_1_ID, button_1),
+            (BUTTON_2_ID, button_2),
+        ],
+        tree: Some(Tree::new(WINDOW_ID)),
+        focus: BUTTON_1_ID,
     }
 }
 
 pub struct NullActionHandler;
 
 impl ActionHandler for NullActionHandler {
-    fn do_action(&self, _request: ActionRequest) {}
+    fn do_action(&mut self, _request: ActionRequest) {}
+}
+
+struct SimpleActivationHandler;
+
+impl ActivationHandler for SimpleActivationHandler {
+    fn request_initial_tree(&mut self) -> Option<TreeUpdate> {
+        Some(get_initial_state())
+    }
 }
 
 fn scope<F>(f: F) -> Result<()>
@@ -57,9 +59,8 @@ where
 {
     super::scope(
         WINDOW_TITLE,
-        get_initial_state(),
-        BUTTON_1_ID,
-        Box::new(NullActionHandler {}),
+        SimpleActivationHandler {},
+        NullActionHandler {},
         f,
     )
 }
@@ -67,7 +68,7 @@ where
 #[test]
 fn has_native_uia() -> Result<()> {
     scope(|s| {
-        let has_native_uia: bool = unsafe { UiaHasServerSideProvider(s.window) }.into();
+        let has_native_uia: bool = unsafe { UiaHasServerSideProvider(s.window.0) }.into();
         assert!(has_native_uia);
         Ok(())
     })
@@ -91,7 +92,7 @@ fn is_button_2(element: &IUIAutomationElement) -> bool {
 #[test]
 fn navigation() -> Result<()> {
     scope(|s| {
-        let root = unsafe { s.uia.ElementFromHandle(s.window) }?;
+        let root = unsafe { s.uia.ElementFromHandle(s.window.0) }?;
         let walker = unsafe { s.uia.ControlViewWalker() }?;
 
         // The children of the window include the children that we provide,
@@ -160,10 +161,10 @@ fn navigation() -> Result<()> {
         assert!(equal);
 
         let wrapped_child = unsafe { walker.GetFirstChildElement(&button_1_forward) };
-        assert_eq!(Err(Error::OK), wrapped_child);
+        assert_eq!(Err(Error::empty()), wrapped_child);
 
         let wrapped_child = unsafe { walker.GetLastChildElement(&button_1_forward) };
-        assert_eq!(Err(Error::OK), wrapped_child);
+        assert_eq!(Err(Error::empty()), wrapped_child);
 
         Ok(())
     })
@@ -173,7 +174,10 @@ fn navigation() -> Result<()> {
 fn focus() -> Result<()> {
     scope(|s| {
         let (focus_event_handler, received_focus_event) = FocusEventHandler::new();
-        unsafe { s.uia.AddFocusChangedEventHandler(None, focus_event_handler) }?;
+        unsafe {
+            s.uia
+                .AddFocusChangedEventHandler(None, &focus_event_handler)
+        }?;
 
         s.show_and_focus_window();
         let focus_from_event = received_focus_event.wait(is_button_1);
